@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -20,9 +20,10 @@ from __future__ import absolute_import, division, print_function
 from collections import defaultdict
 
 import hypothesis.internal.conjecture.utils as cu
-from hypothesis._settings import Phase, note_deprecation
+from hypothesis._settings import Phase
 from hypothesis.control import _current_build_context, assume
 from hypothesis.errors import (
+    HypothesisException,
     NoExamples,
     NoSuchExample,
     Unsatisfiable,
@@ -266,27 +267,20 @@ class SearchStrategy(Generic[Ex]):
         context = _current_build_context.value
         if context is not None:
             if context.data is not None and context.data.depth > 0:
-                note_deprecation(
+                raise HypothesisException(
                     "Using example() inside a strategy definition is a bad "
-                    "idea. It will become an error in a future version of "
-                    "Hypothesis, but it's unlikely that it's doing what you "
-                    "intend even now. Instead consider using "
-                    "hypothesis.strategies.builds() or "
-                    "@hypothesis.strategies.composite to define your strategy."
-                    " See "
-                    "https://hypothesis.readthedocs.io/en/latest/data.html"
+                    "idea. Instead consider using hypothesis.strategies.builds() "
+                    "or @hypothesis.strategies.composite to define your strategy."
+                    " See https://hypothesis.readthedocs.io/en/latest/data.html"
                     "#hypothesis.strategies.builds or "
                     "https://hypothesis.readthedocs.io/en/latest/data.html"
                     "#composite-strategies for more details."
                 )
             else:
-                note_deprecation(
+                raise HypothesisException(
                     "Using example() inside a test function is a bad "
-                    "idea. It will become an error in a future version of "
-                    "Hypothesis, but it's unlikely that it's doing what you "
-                    "intend even now. Instead consider using "
-                    "hypothesis.strategies.data() to draw "
-                    "more examples during testing. See "
+                    "idea. Instead consider using hypothesis.strategies.data() "
+                    "to draw more examples during testing. See "
                     "https://hypothesis.readthedocs.io/en/latest/data.html"
                     "#drawing-interactively-in-tests for more details."
                 )
@@ -355,6 +349,12 @@ class SearchStrategy(Generic[Ex]):
         """
         return FilteredStrategy(condition=condition, strategy=self)
 
+    def do_filtered_draw(self, data, filter_strategy):
+        # Hook for strategies that want to override the behaviour of
+        # FilteredStrategy. Most strategies don't, so by default we delegate
+        # straight back to the default filtered-draw implementation.
+        return filter_strategy.default_do_filtered_draw(data)
+
     @property
     def branches(self):
         # type: () -> List[SearchStrategy[Ex]]
@@ -591,6 +591,9 @@ class MappedSearchStrategy(SearchStrategy):
         ]
 
 
+filter_not_satisfied = UniqueIdentifier("filter not satisfied")
+
+
 class FilteredStrategy(SearchStrategy):
     def __init__(self, strategy, condition):
         super(FilteredStrategy, self).__init__()
@@ -616,6 +619,20 @@ class FilteredStrategy(SearchStrategy):
 
     def do_draw(self, data):
         # type: (ConjectureData) -> Ex
+        result = self.filtered_strategy.do_filtered_draw(
+            data=data, filter_strategy=self
+        )
+        if result is not filter_not_satisfied:
+            return result
+
+        data.note_event("Aborted test because unable to satisfy %r" % (self,))
+        data.mark_invalid()
+        raise AssertionError("Unreachable, for Mypy")  # pragma: no cover
+
+    def note_retried(self, data):
+        data.note_event(lazyformat("Retried draw from %r to satisfy filter", self))
+
+    def default_do_filtered_draw(self, data):
         for i in hrange(3):
             start_index = data.index
             value = data.draw(self.filtered_strategy)
@@ -623,16 +640,13 @@ class FilteredStrategy(SearchStrategy):
                 return value
             else:
                 if i == 0:
-                    data.note_event(
-                        lazyformat("Retried draw from %r to satisfy filter", self)
-                    )
+                    self.note_retried(data)
                 # This is to guard against the case where we consume no data.
                 # As long as we consume data, we'll eventually pass or raise.
                 # But if we don't this could be an infinite loop.
                 assume(data.index > start_index)
-        data.note_event("Aborted test because unable to satisfy %r" % (self,))
-        data.mark_invalid()
-        raise AssertionError("Unreachable, for Mypy")  # pragma: no cover
+
+        return filter_not_satisfied
 
     @property
     def branches(self):
